package com.hexin.android.weituo

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.hexin.android.theme.ThemeManager
import com.hexin.android.ui.NetWorkClinet
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.exception.QueueFullException
import com.hexin.middleware.GlobalHandler
import com.hexin.middleware.MiddlewareProxy
import com.hexin.middleware.ProtocalDef
import com.hexin.middleware.QueueManagement
import com.hexin.middleware.data.StuffBaseStruct
import com.hexin.middleware.data.mobile.StuffTableStruct
import com.hexin.plat.android.R
import com.hexin.util.HexinUtils
import kotlinx.android.synthetic.qs.view_duplicated_code.view.*

class DuplicatedCodeManager: NetWorkClinet, AdapterView.OnItemClickListener {
    var listener: DuplicatedListener? = null
    var  view: View? = null
    var listView: ListView? = null
    var originalCode: String = ""
    var mPopWindow: PopupWindow? = null
    val adapter = MyAdapter(MiddlewareProxy.getHexin())
    var list = mutableListOf<EQBasicStockInfo>()

    override fun receive(struct: StuffBaseStruct?) {
        GlobalHandler.post{
            if (struct is StuffTableStruct) {
                val codes = struct.getData(ProtocalDef.SM_HQ_CODECODE)
                val markets = struct.getData(ProtocalDef.SM_HQ_MARKETTYPE)
                val names = struct.getData(ProtocalDef.SM_HQ_CODENAME)

                if (codes != null && markets != null && names != null && codes.isNotEmpty()
                        && codes.size == markets.size && codes.size == names.size) {
                    if (codes.size == 1) { //仅有一个结果，无重码问题
                        listener?.onResult(codes[0], HexinUtils.toHexString(markets[0]))
                    } else {
                        list.clear()
                        for (i in codes.indices) {
                            list.add(EQBasicStockInfo(names[i], codes[i], markets[i]))
                        }
                        adapter.setData(list)
                        showPopWindow(view)
                        adapter.notifyDataSetChanged()
                    }

                    return@post
                }
            }

            listener?.onResult(originalCode, "")
        }
    }

    override fun onItemClick(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
        if (list.size > position) {
            listener?.onResult(list[position].mStockCode, HexinUtils.toHexString(list[position].mMarket))
        }

        mPopWindow?.dismiss()
    }

    override fun request() {

    }

    fun checkDuplicated(code: String, view: View, listener: DuplicatedListener) {
        this.listener = listener
        this.view = view
        originalCode = code
        release() //每次请求时，都更换instanceId，防止上一次请求返回数据干扰判断

        val requestText = "match_stock=$code\nstartlimit=0\nendlimit=20"
        MiddlewareProxy.request(8001, 8002, getInstanceId(), requestText)
    }

    fun release() {
        QueueManagement.remove(this)
    }

    private fun showPopWindow(mView: View?) {
        mView?.let {mView ->
            if (listView == null) {
                listView = ListView(mView.context).apply {
                    setBackgroundResource(ThemeManager.getDrawableRes(mView.context, R.drawable.jiaoyi_login_pulldown_listview_bg))
                    divider = ColorDrawable(ThemeManager.getColor(mView.context, R.color.list_divide_color))
                    dividerHeight = 1
                    setSelector(ThemeManager.getDrawableRes(mView.context, R.drawable.list_item_pressed_bg))
                    adapter = this@DuplicatedCodeManager.adapter
                    onItemClickListener = this@DuplicatedCodeManager
                }
            }


            mPopWindow = PopupWindow(mView.context).apply {
                width = if (mView.width > 500) mView.width else 500
                height = LinearLayout.LayoutParams.WRAP_CONTENT
                setBackgroundDrawable(ColorDrawable())
                contentView = listView
                isOutsideTouchable = true
                isFocusable = true
                showAsDropDown(mView)
            }
        }
    }

    private fun getInstanceId(): Int {
        var instanceId = QueueManagement.INVALID_ID
        try {
            instanceId = QueueManagement.getId(this)
        } catch (e: QueueFullException) {
            e.printStackTrace()
        }

        return instanceId
    }
}

class MyAdapter(val context: Context): BaseAdapter() {
    val list = mutableListOf<EQBasicStockInfo>()

    fun setData(list: MutableList<EQBasicStockInfo>) {
        this.list.clear()
        this.list.addAll(list)
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val content = View.inflate(context, R.layout.view_duplicated_code, null)
        content.stockCode.text = list[position].mStockCode
        content.stockName.text = list[position].mStockName
        return content
    }

    override fun getItem(position: Int): Any {
        return list[position]
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return list.size
    }
}

interface DuplicatedListener {
    fun onResult(code: String, market: String)
}